import React from 'react';
import { QuestionCircleOutlined } from '@ant-design/icons';
import style from './index.less';
import {
  DatePicker,
  Select,
  Button,
  Checkbox,
  Row,
  Col,
  Dropdown,
  Menu,
  TreeSelect,
  Tooltip,
  Modal,
  Input,
  message,
} from 'antd';
const { Option } = Select;
const { TextArea } = Input;
import moment from 'moment';
import 'moment/locale/zh-cn';
import locale from 'antd/lib/date-picker/locale/zh_CN';
import { HomeOutlined, FolderOpenOutlined } from '@ant-design/icons';
moment.locale('zh-cn');

import { i18n } from '../../util/i18n.js';

export class TopBar extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      fclasslist: [],
      rootTreeData: [], //根节点异步树
      treeLoadedKeys: [],
      treeExpandedKeys: [],
      deptTreeData: [],
      requestData: {
        fclass: '',
        root: undefined,
        department: undefined,
        level: '1',
        fisvitual: '0',
        hidedept: '0',
      },
      open: false,
      confirmLoading: false,
      description: '',
      defaultLevelOpt: [],
      levelOpt: [],
    };
  }

  /**
   * 表单值改变
   * @param {*} payload
   */
  handleFormChange(payload) {
    let requestData = { ...this.state.requestData, ...payload };
    this.setState({ requestData });
  }

  onChange = (e) => {
    this.setState({ description: e.target.value });
  };

  /**
   * 弹窗确认
   */
  handleOk = () => {
    const { description, requestData } = this.state;
    if (description.length == 0) {
      return message.error('请填写描述信息');
    }
    this.setState({ confirmLoading: true });
    let api =
      '/api/bs/hrmorganization/orgchart/versionRecord?fclass=' +
      requestData.fclass +
      '&description=' +
      description;
    fetch(api)
      .then((res) => res.json())
      .then((data) => {
        if (data.api_status) {
          this.setState({
            open: false,
            confirmLoading: false,
          });
          message.success('版本记录成功,请重新刷新页面', 2, 3);
        } else {
          message.error('版本记录失败,请联系相关人员处理数据', 2, 3);
        }
      });
  };

  handleExportMenuClick(e) {
    this.props.onExport(e.key == '1' ? 'png' : 'pdf');
  }

  handleExportButtonClick() {
    this.props.onExport('png');
  }

  /**
   * 根节点树数据
   * @param {} parentId
   * @returns
   */
  getNodeTreeNode = (url, merge = true) => {
    fetch(url)
      .then((res) => res.json())
      .then((data) => {
        if (data.api_status) {
          let arr = [];
          if (merge) {
            arr = [...this.state.rootTreeData, ...data.companyTree];
          } else {
            arr = [...data.companyTree];
          }
          arr.map((item, index) => {
            item.icon = <HomeOutlined />;
          });
          this.setState({
            rootTreeData: arr,
          });
        }
      });
  };

  /**
   * 根节点树异步加载
   * @param {} parentId
   * @returns
   */
  onRootLoadData = (treeNode) =>
    new Promise((resolve) => {
      const { id } = treeNode;
      setTimeout(() => {
        const { fclass } = this.state.requestData;
        let api =
          '/api/bs/hrmorganization/orgchart/getSubCompanyTree?subcompany=' +
          id +
          '&fclass=' +
          fclass;
        this.getNodeTreeNode(api);
        resolve(undefined);
      }, 500);
    });

  onRootChange = (value) => {
    const { fclass } = this.state.requestData;
    if (fclass != '0') {
      //顺胜项目二开 增加部门节点树
      this.getDeptTreeData(
        `/api/bs/hrmorganization/orgchart/getDepartmentTree?fclass=${fclass}&subcompany=${value}`,
      );
    }
    this.handleFormChange({ department: undefined, root: value });
  };

  /**
   * 部门节点树
   */
  getDeptTreeData = (url) => {
    fetch(url)
      .then((res) => res.json())
      .then((data) => {
        if (data.api_status) {
          let arr = [...data.departmentTree];
          arr.map((item, index) => {
            item.icon = <FolderOpenOutlined />;
          });
          this.setState({
            deptTreeData: arr,
          });
        }
      });
  };

  onDeptChange = (value, label, extra) => {
    const { deptTreeData } = this.state;
    var opt = this.buildLevelOpt(0);
    const foundItem = deptTreeData.find((item) => item.value === value);
    var level = foundItem ? foundItem.depth : '1';
    if (value != undefined) {
      opt = this.buildLevelOpt(level);
    }
    this.handleFormChange({
      department: value,
      level: level === '1' ? level : `${level + 1}`,
    });
    this.setState({ levelOpt: opt });
  };

  /**
   * 虚拟维度部门层级构建
   */
  buildLevelOpt = (level) => {
    const { language } = this.props;
    let opt = [{ value: '1', label: `${i18n.all[language]}` }];
    for (let index = 1; index <= level; index++) {
      opt.push({
        value: `${index + 1}`,
        label: language == 'cn' ? `${index}级` : `Level ${index}`,
      });
    }
    return opt;
  };

  onTreeExpand = (expandedKeys) => {
    this.setState({
      treeExpandedKeys: expandedKeys,
    });
  };

  componentDidMount() {
    const { language } = this.props;
    const defaultLevelOpt = [
      { value: '1', label: `${i18n.all[language]}` },
      { value: '2', label: language == 'cn' ? '一级' : 'Level 1' },
      { value: '3', label: language == 'cn' ? '二级' : 'Level 2' },
      { value: '4', label: language == 'cn' ? '三级' : 'Level 3' },
    ];
    this.setState({
      levelOpt: defaultLevelOpt,
      defaultLevelOpt: defaultLevelOpt,
    });
    this.getSeatchCondition(this.props.url);
  }

  getSeatchCondition = (url) => {
    fetch(url)
      .then((res) => res.json())
      .then((data) => {
        data.companyTree.map((item, index) => {
          item.icon = <HomeOutlined />;
        });
        this.setState({
          fclasslist: data.fclasslist,
          rootTreeData: data.companyTree,
        });
        this.handleFormChange({ fclass: data.fclass, root: data.root });
        this.getDeptTreeData(
          `/api/bs/hrmorganization/orgchart/getDepartmentTree?fclass=${data.fclass}&subcompany=${data.root}`,
        );
      });
  };

  menu = (
    <Menu
      onClick={this.handleExportMenuClick.bind(this)}
      items={[
        {
          label: `${i18n.exportImage[this.props.language]}`,
          key: '1',
        },
        // {
        //   label: '导出PDF',
        //   key: '2',
        // },
      ]}
    />
  );

  render() {
    const { disabled, type, language } = this.props;
    const {
      rootTreeData,
      open,
      confirmLoading,
      treeExpandedKeys,
      requestData,
      deptTreeData,
      levelOpt,
      defaultLevelOpt,
    } = this.state;
    const { fclass, department } = requestData;

    return (
      <div className={style.topbarWrapper}>
        <Row>
          <Col span={6}>
            {i18n.fclass[language]}：
            <Select
              defaultValue="0"
              style={{ width: 140 }}
              value={fclass}
              onChange={(value) => {
                const requestData = {
                  fclass: value,
                  root: undefined,
                  department: undefined,
                  level: value == '0' ? '2' : '1',
                  fisvitual: '0',
                  hidedept: '0',
                };
                this.handleFormChange(requestData);
                this.setState({
                  rootTreeData: [],
                  deptTreeData: [],
                  levelOpt:
                    value == '0' ? defaultLevelOpt : this.buildLevelOpt(0),
                });
                this.getNodeTreeNode(
                  `/api/bs/hrmorganization/orgchart/getSubCompanyTree?fclass=${value}`,
                  false,
                );
                this.props.changeFclass(requestData);
              }}
            >
              {this.state.fclasslist.map((item) => (
                <Option key={item.key} value={item.id}>
                  {item.companyname}
                </Option>
              ))}
            </Select>
          </Col>
          <Col span={fclass == '0' ? 6 : 8}>
            {i18n.companyRoot[language]}：
            <TreeSelect
              treeDataSimpleMode
              allowClear
              style={{ width: '65%' }}
              value={this.state.requestData.root}
              dropdownStyle={{ maxHeight: 400, overflow: 'auto' }}
              placeholder={i18n.placeholder[language]}
              onChange={this.onRootChange}
              loadData={this.onRootLoadData}
              treeData={rootTreeData}
              treeIcon
            />
          </Col>
          {fclass == '0' ? (
            <Col span={6}>
              <Checkbox
                style={{ marginTop: '5px', marginLeft: 100 }}
                checked={this.state.requestData.hidedept == '1'}
                onChange={(e) =>
                  this.handleFormChange({
                    hidedept: e.target.checked ? '1' : '0',
                  })
                }
              >
                {i18n.hideDept[language]}
              </Checkbox>

              <Tooltip
                title={i18n.tipsDept[language]}
                color="#0082fb"
                placement="rightTop"
              >
                <QuestionCircleOutlined
                  style={{ color: '#0082fb', cursor: 'pointer', fontSize: 16 }}
                />
              </Tooltip>
            </Col>
          ) : (
            ''
          )}
          <Col span={6}>
            {i18n.deptLevel[language]}：
            <Select
              style={{ width: 140 }}
              value={this.state.requestData.level}
              onChange={(value) => this.handleFormChange({ level: value })}
            >
              {levelOpt.map((item) => (
                <Option key={item.value} value={item.value}>
                  {item.label}
                </Option>
              ))}
            </Select>
          </Col>
        </Row>
        <Row style={{ marginTop: '15px' }}>
          {fclass == '0' ? (
            <Col span={6}>
              <Checkbox
                style={{ marginTop: '5px' }}
                checked={this.state.requestData.fisvitual == '1'}
                onChange={(e) =>
                  this.handleFormChange({
                    fisvitual: e.target.checked ? '1' : '0',
                  })
                }
              >
                {i18n.virtual[language]}
              </Checkbox>
              <Tooltip
                title={i18n.tipsVirtual[language]}
                color="#0082fb"
                placement="rightTop"
              >
                <QuestionCircleOutlined
                  style={{ color: '#0082fb', cursor: 'pointer', fontSize: 16 }}
                />
              </Tooltip>
            </Col>
          ) : (
            ''
          )}
          {fclass != '0' ? (
            <Col span={8}>
              {i18n.deptRoot[language]}：
              <TreeSelect
                treeDataSimpleMode
                allowClear
                style={{ width: '75%' }}
                value={department}
                dropdownStyle={{ maxHeight: 400, overflow: 'auto' }}
                placeholder={i18n.placeholder[language]}
                onChange={this.onDeptChange}
                treeData={deptTreeData}
                treeIcon
              />
            </Col>
          ) : (
            ''
          )}
          <Col span={10}>
            <Button
              type="primary"
              style={{ marginRight: '10px' }}
              onClick={() => {
                if (fclass != '0' && department == undefined) {
                  return message.warning(`${i18n.searchRemind[language]}`);
                }
                this.props.onSearch(this.state.requestData);
              }}
            >
              {i18n.search[language]}
            </Button>
            <Dropdown overlay={this.menu}>
              <Button type="primary">{i18n.export[language]}</Button>
            </Dropdown>
          </Col>
        </Row>
      </div>
    );
  }
}
